// bls0_sdl2.c

STATIC void bls_init()
{
        if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_EVENTS) < 0)
		bls_err("bls_init: SDL2 init error");
}

STATIC void bls_quit()
{
	SDL_Quit();
}

STATIC void bls_wait0(int msec)
{
	do {
		SDL_Event ev;
		while (SDL_PollEvent(&ev) != 0) {
			if (ev.type == SDL_QUIT && bls_autoQuit != 0)
				bls_exit0(0);
			if (ev.type == SDL_KEYDOWN) {
				int i = -1;
				SDL_Keycode sym = ((SDL_KeyboardEvent *)&ev)->keysym.sym;
				if (sym == SDLK_RETURN)		i = bls_KEY_ENTER;
				if (sym == SDLK_ESCAPE)		i = bls_KEY_ESC;
				if (sym == SDLK_BACKSPACE)	i = bls_KEY_BACKSPACE;
				if (sym == SDLK_TAB)		i = bls_KEY_TAB;
				if (sym == SDLK_PAGEUP)		i = bls_KEY_PAGEUP;
				if (sym == SDLK_PAGEDOWN)	i = bls_KEY_PAGEDWN;
				if (sym == SDLK_END)		i = bls_KEY_END;
				if (sym == SDLK_HOME)		i = bls_KEY_HOME;
				if (sym == SDLK_LEFT)		i = bls_KEY_LEFT;
				if (sym == SDLK_RIGHT)		i = bls_KEY_RIGHT;
				if (sym == SDLK_UP)		i = bls_KEY_UP;
				if (sym == SDLK_DOWN)		i = bls_KEY_DOWN;
				if (sym == SDLK_INSERT)		i = bls_KEY_INS;
				if (sym == SDLK_DELETE)		i = bls_KEY_DEL;
				if (0x20 <= sym && sym <= 0x7e)	i = sym;
				if (i >= 0) {
					int j;
					for (j = 0; j < bls_MAXWINDOWS; j++) {
						if (bls_table_h[j] != 0 && SDL_GetWindowID(bls_table_h[j]) == ((SDL_KeyboardEvent *)&ev)->windowID)
							break;
					}
					if (j < bls_MAXWINDOWS)
						bls_putKeybuf(bls_table_w[j], i);
				}
			}
		}
		if (msec < 0)
			SDL_Delay(64);
		if (msec > 0) {
			if (msec >= 64) {
				SDL_Delay(64);
				msec -= 64;
			} else {
				SDL_Delay(msec);
				msec = 0;
			}
		}
	} while (msec != 0);
}

STATIC void bls_openWin0(bls_Window *w)
{
        w->hWnd = SDL_CreateWindow(w->title, SDL_WINDOWPOS_UNDEFINED, SDL_WINDOWPOS_UNDEFINED, w->xsiz, w->ysiz, 0);
	w->rndr = SDL_CreateRenderer(w->hWnd, -1, 0);
        w->txtr = SDL_CreateTexture(w->rndr, SDL_PIXELFORMAT_ARGB8888, SDL_TEXTUREACCESS_STREAMING, w->xsiz, w->ysiz);
	bls_table_h[w->table_i] = w->hWnd;
}

STATIC void bls_closeWin0(bls_Window *w)
{
	SDL_DestroyTexture(w->txtr);
	SDL_DestroyRenderer(w->rndr);
	SDL_DestroyWindow(w->hWnd);
	bls_table_h[w->table_i] = 0;
	w->hWnd = 0;
}

STATIC void bls_flushAll0(bls_Window *w)
{
	uint32_t *p = w->buf;
	char *q;
	int linebyte;
	if (SDL_LockTexture(w->txtr, 0, (void **) &q, &linebyte) == 0) {
		if (linebyte == w->xsiz * sizeof (uint32_t))
			memcpy(q, p, w->xsiz * w->ysiz * sizeof (uint32_t));
		else {
			int y;
			for (y = 0; y < w->ysiz; y++) {
				memcpy(q, p, w->xsiz * sizeof (uint32_t));
				p += w->xsiz;
				q += linebyte;
			}
		}
		SDL_UnlockTexture(w->txtr);
	        SDL_RenderCopy(w->rndr, w->txtr, 0, 0);
        	SDL_RenderPresent(w->rndr);
	}
}
